#!/usr/bin/env php
<?php

/**
 * @author Dominique Fournier <dominique@fournier38.fr>
 * @license BSD
 */

declare(strict_types=1);

require_once(__DIR__ . "/../vendor/autoload.php");

// Catch all the PHP error messages
new App\Service\CatchError();

// Set the timezone based on system state
new App\Service\LocalTimezone();

// Get the command line parameters
$getopts = new App\Service\Getopts();

// Resolve the entries
$dns = new App\Service\Dns();
$dns->ipv4($getopts->getIPv4());
$dns->ipv6($getopts->getIPv6());
$resolved = $dns->directDNS($getopts->getNames());

// Send the pings simultaneously and get the analyzed objects
$asyncping = new App\Service\AsyncPing();
$history = new App\Service\History();
$displayFormat =  $getopts->getDisplay();
$displayClass = "App\Service\Display" . $displayFormat;
/** @var App\Service\DisplayInterface $display */
$display = new $displayClass($resolved);

$asyncping->init($resolved, function ($result) use ($history, $display) {
    $history->store($result);
    $display->run($history);
});

$asyncping->run($resolved, function ($result) use ($history, $display) {
    $history->store($result);
    $display->run($history);
    //echo $result . "\n";
});
