#!/usr/bin/env php
<?php

/**
 * Create the .PHAR file containing all the application
 * To Use :             php docs/createPhar.php
 * The result is in :   /tmp/pingo.phar
 */

$srcRoot = realpath(__DIR__ . "/..") ?: "";

$pharFile = "pingo";
$startFileCLI = "bin/pingo.php";
$startFileWeb = "bin/pingo.php";
$exclude = [
    "docs/", "debian/", ".git/", ".gitignore", "composer.json", "composer.lock", "phpcs.xml",
    "phpstan.neon", "README.md", "pingo"
];
$compact = true;

if (file_exists($pharFile)) {
    unlink($pharFile);
}
if (file_exists("$pharFile.phar")) {
    unlink("$pharFile.phar");
}

ini_set("date.timezone", "Europe/Paris");
$version = date("Y-m-d H:i");

// create phar
$phar = new Phar("$pharFile.phar");

$iterator = new RecursiveDirectoryIterator($srcRoot);
$iterator = new RecursiveIteratorIterator($iterator, RecursiveIteratorIterator::SELF_FIRST);
foreach ($iterator as $file) {
    /** @var string $file */
    $relativeFile = substr($file, strlen($srcRoot) + 1);
    if (basename($relativeFile) === "." || basename($relativeFile) === "..") {
        continue;
    }
    if (! is_file($relativeFile)) {
        continue;
    }
    foreach ($exclude as $excl) {
        if (
            substr($excl, -1) === "/" &&
            substr($relativeFile, 0, strlen($excl) - 1) . "/" === $excl
        ) {
            continue 2;
        }
        if (
            basename(dirname($relativeFile)) === $excl ||
            $relativeFile === $excl
        ) {
            continue 2;
        }
    }
    if ($compact) {
        $content = php_strip_whitespace($relativeFile) ?: "";
    } else {
        $content = file_get_contents($relativeFile) ?: "";
    }
    $content = str_replace("__VERSION__", $version, $content);
    $phar->addFromString($relativeFile, $content);
}
$tmp = tempnam("/tmp", "stub-") ?: "";
file_put_contents($tmp, $phar->createDefaultStub(
    $startFileCLI,
    $startFileWeb
));

if ($compact) {
    $phar->setStub("#!/usr/bin/env php\n" . php_strip_whitespace($tmp));
} else {
    $phar->setStub("#!/usr/bin/env php\n" . file_get_contents($tmp));
}
unlink($tmp);

rename("$pharFile.phar", $pharFile);
chmod($pharFile, 0700);

if (file_exists($pharFile)) {
    echo "   File '$pharFile' created\n";
} else {
    echo "   ERROR : Can not create '$pharFile'\n";
}
