<?php

/**
 * @author Dominique Fournier <dominique@fournier38.fr>
 * @license BSD
 */

declare(strict_types=1);

namespace App\Service;

class Display
{
    private int $lengthIPs = 0;
    private int $lengthNames = 0;
    /**
     * @var array<string, array<string>>
     */
    protected array $resolved = [];

    /**
     * Calculate the max size of the name + ip address do be displayed
     *
     * @param array<string, array<string>> $resolved
     */
    public function __construct(array $resolved)
    {
        $this->resolved = $resolved;
        foreach ($this->resolved as $name => $ips) {
            $this->lengthNames = max($this->lengthNames, strlen($name));
            foreach ($ips as $ip) {
                $this->lengthIPs = max($this->lengthIPs, strlen($ip));
            }
        }
    }

    /**
     * Clear the screen
     *
     * @return void
     */
    public function clearScreen(): void
    {
        echo "\033[2J\033[;H\033c";
        echo "\033[38;5;2mDisplay mode : " . substr(strrchr(get_called_class(), "\\") ?: "", 8) . "\033[0m" . PHP_EOL;
    }

    /**
     * Display one line with the associated colors
     *
     * @param ?string $name The host name to ping
     * @param ?string $ip The ip of the host to ping
     * @param ?PingStateInterface $last The state to display
     *
     * @return void
     */
    public function displayLine(?string $name, ?string $ip, ?PingStateInterface $last): void
    {
        $ip = ($ip === null) ? "" : "[$ip]";
        echo sprintf("%{$this->lengthNames}s ", $name);
        echo sprintf("%-" . ($this->lengthIPs + 2) . "s ", $ip);
        if ($last !== null) {
            if ($last instanceof PingStateOK) {
                echo "\033[38;5;2m  OK  \033[0m ";
                echo "(since " . ($last->getEndTime() - $last->getStartTime()) . "s / ";
                echo date("H:i:s", $last->getStartTime()) . ") ";
                echo "RTT=" . sprintf("%5.1f", $last->getTime()) . $last->getTimeunit();
            } elseif ($last instanceof PingStateBad) {
                echo "\033[48;5;1mFAILED\033[0m ";
                echo "(since " . ($last->getEndTime() - $last->getStartTime()) . "s / ";
                echo date("H:i:s", $last->getStartTime()) . ") ";
            } elseif ($last instanceof PingStateError) {
                echo "\033[38;5;5mERROR \033[0m ";
                echo "(since " . ($last->getEndTime() - $last->getStartTime()) . "s / ";
                echo date("H:i:s", $last->getStartTime()) . ") ";
                echo "\033[38;5;5m" . $last->getError() . "\033[0m ";
            } elseif ($last instanceof PingStateNull) {
                echo "       ";
                echo "(since " . ($last->getEndTime() - $last->getStartTime()) . "s / ";
                echo date("H:i:s", $last->getStartTime()) . ") ";
            }
            //echo $last;
        }
        echo PHP_EOL;
    }
}
