<?php

/**
 * @author Dominique Fournier <dominique@fournier38.fr>
 * @license BSD
 */

declare(strict_types=1);

namespace App\Service;

/**
 * Store the changes in history, by IP
 */
class History
{
    /**
     * History format :
     * $history[$ip][] = change
     *
     * @var array<string, array<PingStateInterface>>
     */
    private array $history = [];

    /**
     * Transition format :
     * $transition[$ip][] = change
     *
     * @var array<string, array<PingStateInterface>>
     */
    private array $transition = [];

    /**
     * @param PingStateInterface $state The current states to store
     *
     * @return $this
     */
    public function store(PingStateInterface $state): self
    {
        $ip = $state->getIP();
        $state->setEndTime(time());
        if (! key_exists($ip, $this->history)) {
            //$state->setStartTime(time());
        } else {
            /** @var PingStateInterface */
            $last = end($this->history[$ip]);
            if (get_class($last) === get_class($state) || $last instanceof PingStateNull) {
                // Maintain the state
                $state->setStartTime($last->getStartTime());
            } else {
                // The state change : store the new time
                //$state->setStartTime(time());
            }
        }
        $this->history[$ip][] = $state;

        if (! key_exists($ip, $this->transition)) {
        } else {
            /** @var PingStateInterface */
            $last = end($this->transition[$ip]);
            if (get_class($last) === get_class($state) || $last instanceof PingStateNull) {
                // Maintain the state
                $state->setStartTime($last->getStartTime());
                array_pop($this->transition[$ip]);
            } else {
                // The state change : store the new time
            }
        }
        $this->transition[$ip][] = $state;

        return $this;
    }

    /**
     * Return the last states stored
     * @param string $ip The IP to get the last history entry
     *
     * @return ?PingStateInterface
     */
    public function getLast(string $ip): ?PingStateInterface
    {
        if (! key_exists($ip, $this->transition)) {
            return null;
        }
        /** @var PingStateInterface */
        return end($this->transition[$ip]);
    }

    /**
     * Return all the states stored
     * @param string $ip The IP to get the last history entry
     *
     * @return array<PingStateInterface>|null
     */
    public function getAllTransitions(string $ip): ?array
    {
        return $this->transition[$ip] ?? null;
    }

    /**
     * Return all the details stored
     * @param string $ip The IP to get the last history entry
     *
     * @return ?array<PingStateInterface>
     */
    public function getAllDetails(string $ip): ?array
    {
        return $this->history[$ip] ?? null;
    }
}
